-- Variables
local uniquealyzer = 1;
local checkButtons = {};
local btnCounter = 0;
local itemSlotBtnCounter = 0;

-- Globals
BD_DATA = {
	bg = [[Interface\Tooltips\UI-Tooltip-Background]],
	edge = [[Interface\Buttons\WHITE8X8]],
	bgBorder = [[Interface/Tooltips/UI-Tooltip-Background]],
	edgeBorder = [[Interface/Tooltips/UI-Tooltip-Border]]
};
	
DLU_BD = {
	primary = { bgFile = BD_DATA.bg, tile = true, tileSize = 16, edgeFile = BD_DATA.edge, edgeSize = 1 },
	secondary = { bgFile = BD_DATA.bg, tile = false, tileSize = 4, edgeFile = BD_DATA.edge, edgeSize = .75 },
	border = {bgFile = BD_DATA.bgBorder, tile = false, tileSize = 0, edgeFile = BD_DATA.edgeBorder, edgeSize = 1, insets = { left = -1, right = -1, top = -1, bottom = -1}}
};

-- Locals
local function getBackdropAlpha(frame)
	local _, _, _, a = frame:GetBackdropColor();
	return tonumber(format("%.1f", a));
end

local function hightlight(button)
	local alpha = getBackdropAlpha(button);
	if (alpha ~= .5) then
		button:SetBackdropColor(1,1,1,.4);
	end
end
local function unHightlight(button)
	local alpha = getBackdropAlpha(button);
	if (alpha ~= .5) then
		button:SetBackdropColor(1,1,1,.2);
	end
end

local function HideTooltip(self)
	if (self.statusBar ~= nil) then
		local r,g,b,a = self.statusBar:GetStatusBarColor();
		self.statusBar:SetStatusBarColor(r,g,b,1);
	end
	
	GameTooltip:Hide();
end

-- Methods
function DLU.createTextRoot(text, alignment, parent, x_loc, y_loc, fontType)
	local title = parent:CreateFontString(nil, "ARTWORK", fontType);
	title:SetPoint(alignment, parent, alignment, x_loc, y_loc);
	title:SetText(text);
	return title;
end

function DLU.createTextChild(text, parent, below, x_loc, y_loc, fontType)
	local textChildNode = parent:CreateFontString(nil, "ARTWORK", fontType);
	textChildNode:SetPoint("TOPLEFT", below, "BOTTOMLEFT", x_loc, y_loc);
	textChildNode:SetText(text);
	return textChildNode;
end

function DLU.createCheckbutton(parent, x_loc, y_loc, displayname, option)
	uniquealyzer = uniquealyzer + 1;
	
	local checkbutton = CreateFrame("CheckButton", "checkbutton_0" .. uniquealyzer, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:ClearAllPoints()
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	getglobal(checkbutton:GetName() .. 'Text'):SetText(displayname);
	
	-- Option checkbox
	if (option) then
		checkButtons[option] = checkbutton;
	end

	return checkbutton;
end

function DLU.createEditBox(parent, alignment, xOffset, yOffset, height, width)
	local editBox = CreateFrame("EditBox", nil, parent, BackdropTemplateMixin and "BackdropTemplate");
	editBox:SetBackdrop({
		bgFile = "",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = "true",
		tileSize = 32,
		edgeSize = 10,
		insets = {left = 3, right = 3, top = 3, bottom = 3}
	});
	editBox:SetFontObject(ChatFontNormal);
	editBox:SetAutoFocus(false);
	editBox:SetPoint(alignment, parent, alignment, xOffset, yOffset);
	editBox:SetSize(width, height);
	editBox:SetTextInsets(5,0,0,0);
	
	editBox:SetScript("OnEscapePressed", editBox.ClearFocus);
	
	return editBox;
end

function DLU.createStandardButton(parent, text, width, height, tooltip, x_loc, y_loc)
	local button = CreateFrame("Button", "DLUButton" .. btnCounter, parent, "UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", x_loc, y_loc);
    button:SetText(text);
	
	if (tooltip ~= nil) then
		button.tooltipText = tooltip;
	end
    
    button:SetWidth(width);
    button:SetHeight(height);
	btnCounter = btnCounter + 1;
	return button;
end

function DLU.createItemSlotButton(parent, alignment, name)
	if (name == nil) then
		name = "DLUItemSlotBtn" .. itemSlotBtnCounter;
	end

	--local btn = CreateFrame("Button", name, parent or UIParent, "ItemButtonTemplate");
	local btn = CreateFrame("ItemButton", name, parent or UIParent);
    btn:SetSize(37,37);
    btn:SetPoint(alignment, parent or 0, alignment);

	itemSlotBtnCounter = itemSlotBtnCounter + 1;

	return btn;
end

function DLU.FindSettingsFrame(name)
	for k,v in ipairs(DLU.Settings.Frames) do
		if (v.name == name) then
			return v;
		end
	end
end

-- DLUHQ
function DLU.CreatePanel(width, height, parentFrame, yOffset)
	local frame = CreateFrame("Frame", nil, parentFrame, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetSize(width, height);
	frame:SetPoint("TOPLEFT", parentFrame, "TOPLEFT", 0, yOffset);
	frame:SetBackdrop(DLU_BD.secondary);
	frame:SetBackdropColor(0,0,0,.9);
	frame:SetBackdropBorderColor(1,1,1,0);
	return frame;
end

function DLU.CreateLabel(text, fontSize, alignment, parentFrame)
	if (parentFrame == nil) then
		return
	end

	local label = parentFrame:CreateFontString();
	label:SetFont("Fonts\\FRIZQT__.TTF", fontSize);
	label:SetPoint(alignment, parentFrame, alignment, 0, 0);
	label:SetJustifyH("LEFT");
    label:SetJustifyV("TOP");
    label:SetText(text);
    label:SetTextColor(1, 1, 1, 1);
    return label;
end

function DLU.CreateButton(width, height, alignment, parentFrame, xOffset, yOffset)
	if (parentFrame == nil) then
		return
	end

	xOffset = xOffset or 0;
	yOffset = yOffset or 0;

	local button = CreateFrame("Button", nil, parentFrame, BackdropTemplateMixin and "BackdropTemplate");
	button:SetPoint(alignment, parentFrame, alignment, xOffset, yOffset);
	button:SetSize(width, height);
	button:SetBackdrop(DLU_BD.secondary);
	button:SetBackdropColor(1,1,1,.2);
	button:SetBackdropBorderColor(1,1,1,0);
	button:SetScript("OnEnter", hightlight);
	button:SetScript("OnLeave", unHightlight);
	return button;
end

function DLU.CreateStatusBar(parent, width, height, position, xOffset, yOffset, color, hover)
	local frame = CreateFrame("StatusBar", nil, parent);
	frame:SetSize(width, height);
	frame:SetPoint(position, parent, position, xOffset, yOffset);
	frame:SetStatusBarTexture([[Interface\BUTTONS\WHITE8X8.blp]]);
	frame:SetStatusBarColor(color.r, color.g, color.b, color.a);
	frame:SetMinMaxValues(0,100);
	frame:SetScript("OnEnter", hover);
	frame:SetScript("OnLeave", HideTooltip);
	
	return frame;
end

function DLU.CreateBar(Parent, OnEnter, OnClick, width, height, position, xOffset, yOffset, color)
	local fonts = DLU.GetFontSizes();
	
	-- Button
	local bar = CreateFrame("Button", nil, Parent, BackdropTemplateMixin and "BackdropTemplate");
	bar:SetPoint(position, Parent, position, xOffset, yOffset);
	bar:SetSize(width, height);
	bar:SetScript("OnEnter", OnEnter);
	bar:SetScript("OnLeave", HideTooltip);
	if (OnClick ~= nil) then
		bar:SetScript("OnClick", OnClick);
	end
	
	bar:SetBackdrop(DLU_BD.border);
	bar:SetBackdropColor(0,0,0,.6);
	
	-- Status bar
	bar.statusBar = CreateFrame("StatusBar", nil, bar);
	
	bar.statusBar:SetPoint('TOPLEFT', bar, 'TOPLEFT', 0, 0);
	bar.statusBar:SetPoint('BOTTOMRIGHT', bar, 'BOTTOMRIGHT', 0, 0);
	
	bar.statusBar:SetStatusBarTexture([[Interface\BUTTONS\WHITE8X8.blp]]);
	bar.statusBar:SetStatusBarColor(color.r, color.g, color.b, color.a);
	bar.statusBar:SetMinMaxValues(0,100);
	
	-- Text
	bar.text = DLU.CreateLabel("", fonts.small, "CENTER", bar.statusBar);
	
	return bar;
end

function DLU.CreateBorder(parent)
	local frame = CreateFrame("Frame", nil, parent, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetPoint("TOPLEFT", parent, "TOPLEFT", -.5, .5);
	frame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", .5, -.5);
	frame:SetBackdrop(DLU_BD.border);
	frame:SetBackdropColor(0,0,0,.6);
	frame:SetFrameLevel(parent:GetFrameLevel());
	return frame;
end

function DLU.ChangeStatusBarColor(statusbar, newColor)
	statusbar:SetStatusBarColor(newColor.r, newColor.g, newColor.b, .6);
end

function DLU.GetWidth()
	local widths = {
		DLUHQ_MIN = 220
	};
	
	local name = DLUHQ:GetName();
	if (DLU_UI and DLU_UI[name] and DLU_UI[name].width) then
		if (tonumber(DLU_UI[name].width) < widths.DLUHQ_MIN) then
			DLU_UI[name].width = widths.DLUHQ_MIN;
		end
		
		widths.DLUHQ = DLU_UI[name].width;
	else
		widths.DLUHQ = 400;
	end

	return widths;
end

function DLU.GetHeight()
	local heights = {
		Nav = 25,
		DLUHQ_MIN = 140
	};
	
	local name = DLUHQ:GetName();
	if (DLU_UI and DLU_UI[name] and DLU_UI[name].height) then
		if (tonumber(DLU_UI[name].height) < heights.DLUHQ_MIN) then
			DLU_UI[name].height = heights.DLUHQ_MIN;
		end
		
		heights.DLUHQ = DLU_UI[name].height;
	else
		heights.DLUHQ = 150;
	end
	
	return heights;
end

function DLU.GetPaddings()
	return {
		Normal = 10,
		Top = 22
	};
end

function DLU.GetFontSizes()
	return {
		small = 10,
		medium = 12
	};
end