local addonName, addonData = ...;

DLU_UI = DLU_UI or {};

local settingsFrame = CreateFrame("Frame");
local hqFrame;

local useNewUI = DLU.UseNewUI();

local function ChangeRepFormat(self, arg1, arg2, checked)
	DLUSettings["repFormat"] = arg1;
	-- Update the text; if we merely wanted it to display newValue, we would not need to do this
	if (not checked) then
		UIDropDownMenu_SetText(DLU_RepFormating, DLUSettings["repFormat"]);
		DLU.UpdateReputationUI();
	end
	-- Because this is called from a sub-menu, only that menu level is closed by default.
	CloseDropDownMenus();
end

local function CreateQuestionText(text, panel, relativeElement, xOffset, yOffset, font)
	local question = DLU.createTextChild(text, panel, relativeElement, xOffset, yOffset, font);
	question:SetTextScale(1.2);
	question:SetTextColor(255,255,0);

	return question;
end

local function CreateAnswerText(text, panel, relativeElement, xOffset, yOffset, font, textWidth)
	local answer = DLU.createTextChild(text, panel, relativeElement, xOffset, yOffset, font);
	answer:SetWordWrap(true);
	answer:SetWidth(textWidth);
	answer:SetJustifyH("LEFT");

	return answer;
end

local function ShowSimpleTooltip(element, text)
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(element, 'ANCHOR_CURSOR', 0, -4);
	GameTooltip:AddLine(text);
	GameTooltip:Show();
end

local function HideTooltip()
	GameTooltip:Hide();
end

local function GetReputationOptions()
	local container = Settings.CreateDropDownTextContainer()
	container:Add(1, "Current / Total")
	container:Add(2, "Left (%left)")
	return container:GetData()
end

local function HideDefaultBars()
	if (DLUSettings.hideDefaultBars) then
		StatusTrackingBarManager:Hide();
	else
		StatusTrackingBarManager:Show();
	end
end

local function CreateGlobalSettingsUI()
	if (useNewUI) then
		local category = Settings.GetCategory(addonName);

		local subcategory, layout = Settings.RegisterVerticalLayoutSubcategory(category, addonData.L.AccountSettingsHeader);
	
		do
			local name = addonData.L.ShowLoadMessageText;
			local tooltip = addonData.L.ShowLoadMessageTooltip;
			local defaultValue = false;
			
			local setting = Settings.RegisterProxySetting(subcategory, "showLoadMessage", DLUSettings, type(defaultValue), name, defaultValue);
			Settings.CreateCheckBox(subcategory, setting, tooltip);
		end
	
		do
			local name = addonData.L.ShowDLUHQText;
			local tooltip = addonData.L.ShowDLUHQTooltip;
			local defaultValue = false;
			
			local setting = Settings.RegisterProxySetting(subcategory, "showDLUHQOnLogon", DLUSettings, type(defaultValue), name, defaultValue);
			Settings.CreateCheckBox(subcategory, setting, tooltip);
		end
	
		--[[
		do
			local defaultValue = 1
			local name = "Reputation format";
			local tooltip = "Changes the format for showing reputation in DLU HQ."
	
			local setting = Settings.RegisterProxySetting(subcategory, "repFormat", DLUSettings, type(defaultValue), name, defaultValue);
			Settings.CreateDropDown(subcategory, setting, GetReputationOptions, tooltip);
		end
		]]
	
		Settings.RegisterCategory(subcategory);
	else
		local accountSettingsFrame = CreateFrame("Frame");
		accountSettingsFrame.name = addonData.L.AccountSettingsHeader;
		accountSettingsFrame.parent = addonName;

		InterfaceOptions_AddCategory(accountSettingsFrame);

		local title = DLU.createTextRoot(accountSettingsFrame.name, "TOPLEFT", accountSettingsFrame, 16, -16, "GameFontNormalLarge");

		local checkBox = DLU.createCheckbutton(accountSettingsFrame, 10, -36, addonData.L.ShowLoadMessageText, 0);
		checkBox:SetChecked(DLUSettings.showLoadMessage);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUSettings.showLoadMessage = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
			else
				DLUSettings.showLoadMessage = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.ShowLoadMessageText, settingText));
		end);

		checkBox = DLU.createCheckbutton(accountSettingsFrame, 10, -56, addonData.L.ShowDLUHQText, 0);
		checkBox:SetChecked(DLUSettings.showDLUHQOnLogon);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUSettings.showDLUHQOnLogon = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
			else
				DLUSettings.showDLUHQOnLogon = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.ShowDLUHQText, settingText));
		end);

		checkBox = DLU.createCheckbutton(accountSettingsFrame, 10, -76, addonData.L.EnableSellGreyButtonText, 0);
		checkBox:SetChecked(DLUSettings.enableSellGreyButton);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUSettings.enableSellGreyButton = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
				DLU.CheckSellGreyButtonVisibility();
			else
				DLUSettings.enableSellGreyButton = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
				DLU.CheckSellGreyButtonVisibility();
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.EnableSellGreyButtonText, settingText));
		end);

		checkBox = DLU.createCheckbutton(accountSettingsFrame, 10, -96, addonData.L.HideDefaultBarsText, 0);
		checkBox:SetChecked(DLUSettings.hideDefaultBars);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUSettings.hideDefaultBars = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
				HideDefaultBars();
			else
				DLUSettings.hideDefaultBars = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
				HideDefaultBars();
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.HideDefaultBarsText, settingText));
		end);
	end
end

local function CreateCharacterSettingsUI()
	if (useNewUI) then
		local category = Settings.GetCategory(addonName);

		local subcategory, layout = Settings.RegisterVerticalLayoutSubcategory(category, addonData.L.CharacterSettingsHeader);
	
		do
			local name = addonData.L.AutoRepairText;
			local tooltip = "Enables auto-repair feature.";
			local defaultValue = false;
			
			local setting = Settings.RegisterProxySetting(subcategory, "autoRepair", DLUCharacterSettings, type(defaultValue), name, defaultValue);
			Settings.CreateCheckBox(subcategory, setting, tooltip);
		end
	
		Settings.RegisterCategory(subcategory);
	else
		local characterSettingsFrame = CreateFrame("Frame");
		characterSettingsFrame.name = addonData.L.CharacterSettingsHeader;
		characterSettingsFrame.parent = addonName;

		InterfaceOptions_AddCategory(characterSettingsFrame);

		local title = DLU.createTextRoot(characterSettingsFrame.name, "TOPLEFT", characterSettingsFrame, 16, -16, "GameFontNormalLarge");

		local checkBox = DLU.createCheckbutton(characterSettingsFrame, 10, -36, addonData.L.AutoRepairText, 0);
		checkBox:SetChecked(DLUCharacterSettings.autoRepair);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUCharacterSettings.autoRepair = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
			else
				DLUCharacterSettings.autoRepair = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.AutoRepairText, settingText));
		end);
		checkBox:SetScript("OnEnter", function(self)
			ShowSimpleTooltip(self, addonData.L.AutoRepairTooltip);
		end);
		checkBox:SetScript("OnLeave", HideTooltip);

		checkBox = DLU.createCheckbutton(characterSettingsFrame, 10, -56, addonData.L.QuestingModeText, 0);
		checkBox:SetChecked(DLUCharacterSettings.questingMode);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUCharacterSettings.questingMode = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
			else
				DLUCharacterSettings.questingMode = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.QuestingModeText, settingText));
		end);
		checkBox:SetScript("OnEnter", function(self)
			ShowSimpleTooltip(self, addonData.L.QuestingModeTooltip)
		end);
		checkBox:SetScript("OnLeave", HideTooltip);

		checkBox = DLU.createCheckbutton(characterSettingsFrame, 10, -76, addonData.L.SkipCinematicText, 0);
		checkBox:SetChecked(DLUCharacterSettings.skipCinematics);
		checkBox:SetScript("OnClick", function(self)
			local settingText;
			
			if (self:GetChecked()) then
				DLUCharacterSettings.skipCinematics = true;
				settingText = DLU.colorizeString(addonData.L.Enabled, "green");
			else
				DLUCharacterSettings.skipCinematics = false;
				settingText = DLU.colorizeString(addonData.L.Disabled, "red");
			end
			
			DLU.createMessage(format(addonData.L.Option .. "!", addonData.L.SkipCinematicText, settingText));
		end);
		checkBox:SetScript("OnEnter", function(self)
			ShowSimpleTooltip(self, addonData.L.SkipCinematicTooltip)
		end);
		checkBox:SetScript("OnLeave", HideTooltip);
	end
end

local function SetupSettingsUI()
	local panel = CreateFrame("Frame", "DLUOptionsPanel", InterfaceOptionsFramePanelContainer);
	-- Option panel name
	panel.name = addonName;
	-- Add info to panel
	local author = GetAddOnMetadata(addonName, "Author");
	local addonVersion = GetAddOnMetadata(addonName, "Version");
	
	local title = DLU.createTextRoot(addonName, "TOPLEFT", panel, 16, -16, "GameFontNormalLarge");
	local authorText = DLU.createTextChild(DLU.colorizeString(addonData.L.MadeBy, "red") .. ": " .. author, panel, title, 0, -8, "GameFontHighlightSmall");
	local versionText = DLU.createTextChild(DLU.colorizeString(addonData.L.Version, "red") .. ": ".. addonVersion, panel, authorText, 0, -8, "GameFontHighlightSmall");
	local descriptionText = DLU.createTextChild(DLU.colorizeString(addonData.L.Description, "red") .. ": " .. addonData.L.AddonNote, panel, versionText, 0, -8, "GameFontHighlightSmall");

	if (useNewUI) then
		-- Add reset button
		local fixAllErrors = DLU.createStandardButton(panel, addonData.L.ResetAllSettings, 160, 22, addonData.L.ResetAllSettingsTooltip, 10, 30);
		fixAllErrors:SetScript("OnClick", function()
			DLU.resetAddonOptions();
			local showsErrors = GetCVar("ScriptErrors");
			if (showsErrors == "1") then
				SetCVar("ScriptErrors", "0");
			end

			SlashCmdList.RELOADUI();
		end);

		local fixCharacterErrors = DLU.createStandardButton(panel, addonData.L.ResetCharacterSettings, 160, 22, addonData.L.ResetCharacterSettingsTooltip, SettingsPanel.Container.SettingsCanvas:GetWidth() - 170, 30);
		fixCharacterErrors:SetScript("OnClick", function()
			DLU.ResetCharacterSettings();
			local showsErrors = GetCVar("ScriptErrors");
			if (showsErrors == "1") then
				SetCVar("ScriptErrors", "0");
			end

			SlashCmdList.RELOADUI();
		end);
	else
		local fixAllErrors = DLU.createStandardButton(panel, addonData.L.ResetAllSettings, 160, 22, addonData.L.ResetAllSettingsOldTooltip, 10, 30);
		fixAllErrors:SetScript("OnClick", function()
			DLU.resetAddonOptions();
			SlashCmdList.RELOADUI();
		end);

		local fixCharacterErrors = DLU.createStandardButton(panel, addonData.L.ResetCharacterSettings, 160, 22, addonData.L.ResetCharacterSettingsOldTooltip, SettingsPanel.Container.SettingsCanvas:GetWidth() - 170, 30);
		fixCharacterErrors:SetScript("OnClick", function()
			DLU.ResetCharacterSettings();
			SlashCmdList.RELOADUI();
		end);
	end
	
	-- Add panel to addon options
	InterfaceOptions_AddCategory(panel);

	-- Global settings
	CreateGlobalSettingsUI();

	-- FAQ
	local faqPanel = CreateFrame("Frame");
	faqPanel.name = addonData.L.FAQHeader;
	faqPanel.parent = addonName;

	local interfacePanelWidth = SettingsPanel.Container.SettingsCanvas:GetWidth();
	local title = DLU.createTextRoot(faqPanel.name, "TOPLEFT", faqPanel, 16, -16, "GameFontNormalLarge");

	local question = CreateQuestionText(addonData.L.FAQMissingFeaturesHeader, faqPanel, title, 0, -20, "GameFontHighlightSmall");
	local answer = CreateAnswerText(addonData.L.FAQMissingFeaturesAnswer, faqPanel, question, 0, -20, "GameFontHighlightSmall", interfacePanelWidth - 20);

	question = CreateQuestionText(addonData.L.FAQMountUpHeader, faqPanel, answer, 0, -20, "GameFontHighlightSmall");
	answer = CreateAnswerText(addonData.L.FAQMountUpAnswer, faqPanel, question, 0, -20, "GameFontHighlightSmall", interfacePanelWidth - 20);

	question = CreateQuestionText(addonData.L.FAQFixErrorHeader, faqPanel, answer, 0, -20, "GameFontHighlightSmall");
	answer = CreateAnswerText(addonData.L.FAQFixErrorAnswer, faqPanel, question, 0, -20, "GameFontHighlightSmall", interfacePanelWidth - 20);

	if (useNewUI) then
		local question = CreateQuestionText(addonData.L.AddonBlockedError, faqPanel, title, 0, -20, "GameFontHighlightSmall");
		local answer = CreateAnswerText(addonData.L.AddonBlockedErrorAnswer, faqPanel, question, 0, -20, "GameFontHighlightSmall", interfacePanelWidth - 20);
	
		question = CreateQuestionText(addonData.L.AddonWhyChangeSettingsUI, faqPanel, answer, 0, -20, "GameFontHighlightSmall");
		answer = CreateAnswerText(addonData.L.AddonWhyChangeSettingsUIAnswer, faqPanel, question, 0, -10, "GameFontHighlightSmall", interfacePanelWidth - 20);
	
		question = CreateQuestionText(addonData.L.AddonBlockedErrorSolution, faqPanel, answer, 0, -20, "GameFontHighlightSmall");
		answer = CreateAnswerText(addonData.L.AddonBlockedErrorSolutionAnswer, faqPanel, question, 0, -10, "GameFontHighlightSmall", interfacePanelWidth - 20);
	end

	CreateCharacterSettingsUI();

	table.insert(DLU.Settings.Frames, faqPanel);
	
	-- Add all sub setting frames
	for k,v in ipairs(DLU.Settings.Frames) do
		InterfaceOptions_AddCategory(v);
	end
end

local function stopUIMoving(self)
	self:StopMovingOrSizing();
	local name = self:GetName();
	
	if (DLU_UI[name]) then
		DLU_UI[name]["xPos"] = self:GetLeft();
		DLU_UI[name]["yPos"] = self:GetBottom();
	else
		DLU_UI[name] = {};
		DLU_UI[name].xPos = self:GetLeft();
		DLU_UI[name].yPos = self:GetBottom();
	end
end

local function toggleContainer()
	local isVisible = DLUHQ.Container:IsVisible();
	local name = DLUHQ:GetName();
	
	if (not DLU_UI[name]) then
		DLU_UI[name] = {};
	end
	
	if (isVisible) then
		DLUHQ.Container:Hide();
		DLUHQ.minimize.Label:SetText("+");
	else
		DLUHQ.Container:Show();
		DLUHQ.minimize.Label:SetText("-");
	end
	
	DLU_UI[name].minimized = isVisible;
end

local function togglePanel(shownPanel)
	for k, v in pairs(DLUHQ.Panels) do
		if (k == shownPanel) then		
			DLUHQ.navigation[k]:SetBackdropColor(1,1,1,.5);
			DLUHQ.Panels[shownPanel]:Show();
			DLU_UI["shownPanel"] = shownPanel;
		else
			DLUHQ.navigation[k]:SetBackdropColor(1,1,1,.2);
			DLUHQ.Panels[k]:Hide();
		end
	end
end

local function CreateDLUHQ()
	local widths = DLU.GetWidth();
	local heights = DLU.GetHeight();
	local fonts = DLU.GetFontSizes();
	
	local height = heights.DLUHQ;
	local navHeight = heights.Nav;
	
	local width = widths.DLUHQ;
	
	DLUHQ:SetSize(width, 20);
	DLUHQ:SetBackdrop(DLU_BD.primary);
	
	-- Location
	local name = DLUHQ:GetName();
	if (DLU_UI[name] and DLU_UI[name].xPos) then
		DLUHQ:SetPoint("BOTTOMLEFT", DLU_UI[name].xPos, DLU_UI[name].yPos);
	else
		DLUHQ:SetPoint("CENTER", UIParent, "CENTER", 0, (height + 20 + 25) / 2);
	end
	
	DLUHQ:SetBackdropColor(0,0,0,1);
	DLUHQ:SetBackdropBorderColor(.7,.7,.7,1);
	DLUHQ:SetClampedToScreen(true);
	DLUHQ:SetMovable(true);
    DLUHQ:EnableMouse(true);
	DLUHQ:RegisterForDrag("LeftButton");

	DLUHQ:SetScript("OnDragStart", DLUHQ.StartMoving);
    DLUHQ:SetScript("OnDragStop", stopUIMoving);
	
	-- Title
	DLUHQ.title = DLUHQ:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	DLUHQ.title:SetPoint("TOP", 0, -2);
	DLUHQ.title:SetText(format("%s HQ", "DLU"));
	
	-- Settings btn
	DLUHQ.settings = DLU.CreateButton(16, 16, "LEFT", DLUHQ);
	DLUHQ.settings.Label = DLU.CreateLabel("?", 10, "CENTER", DLUHQ.settings);
	DLUHQ.settings:SetScript("OnClick", function()
		InterfaceOptionsFrame_OpenToCategory(addonName);
	end);
	
	-- Minimize button
	DLUHQ.minimize = DLU.CreateButton(16, 16, "RIGHT", DLUHQ, -20);
	DLUHQ.minimize.Label = DLU.CreateLabel("-", 10, "CENTER", DLUHQ.minimize);
	DLUHQ.minimize:SetScript("OnClick", toggleContainer);
	
	-- Close button
	DLUHQ.close = DLU.CreateButton(16, 16, "RIGHT", DLUHQ);
	DLUHQ.close.Label = DLU.CreateLabel("X", 10, "CENTER", DLUHQ.close);
	DLUHQ.close:SetScript("OnClick", function()
		DLUHQ:Hide();
	end);
	
	-- Panel Container
	DLUHQ.Container:SetSize(width, height);
	DLUHQ.Container:SetPoint("TOPLEFT", DLUHQ, "TOPLEFT", 0, -20);
	if (DLU_UI[name] and DLU_UI[name].minimized) then
		toggleContainer();
	end
	
	-- Panel Navigation
	DLUHQ.navigation = CreateFrame("Frame", nil, DLUHQ.Container, BackdropTemplateMixin and "BackdropTemplate");
	DLUHQ.navigation:SetSize(width, navHeight);
	DLUHQ.navigation:SetPoint("TOPLEFT", DLUHQ.Container, "TOPLEFT", 0, 0);
	DLUHQ.navigation:SetBackdrop(DLU_BD.secondary);
	DLUHQ.navigation:SetBackdropColor(.7,0,0, 1);
	DLUHQ.navigation:SetBackdropBorderColor(1,1,1,0);

	-- Sorting panels by name alphabetically (for navigation)
	local keys = {};
	for k in pairs(DLUHQ.Panels) do
		table.insert(keys, k);
	end
	table.sort(keys);

	local i = 0;
	for _, k in ipairs(keys) do
		local xOffset = i * 75;

		DLUHQ.navigation[k] = DLU.CreateButton(75, navHeight, "TOPLEFT", DLUHQ.navigation, xOffset);
		DLUHQ.navigation[k].Label = DLU.CreateLabel(k, fonts.small, "CENTER", DLUHQ.navigation[k]);
		
		DLUHQ.navigation[k]:SetScript("OnClick", function()
			togglePanel(k);
		end);

		i = i+1;
	end
	
	-- Navigate to front page by default
	if (DLU_UI["shownPanel"]) then
		togglePanel(DLU_UI["shownPanel"]);
	else
		togglePanel("Character");
	end
end

local function CheckHQVisibility()
	if (not DLUSettings.showDLUHQOnLogon) then
		DLUHQ:Hide();
	end
end

-- Change Character UI
hooksecurefunc("PaperDollFrame_UpdateStats", function()
	DLU.showStats("player");
end)

settingsFrame:RegisterEvent("PLAYER_LOGIN");
settingsFrame:SetScript("OnEvent", function()
	SetupSettingsUI();
	
	CreateDLUHQ();
	CheckHQVisibility();

	HideDefaultBars();
end);